@@ -28,20 +28,49 @@
 #include "core/gcs-color.h"
 #ifdef HAVE_GCONFMM
 #include <gconfmm/client.h>
+#include <gconfmm/value.h>
 #endif // HAVE_GCONFMM
 
 namespace gcs
 {
+    // gconf recommends storing enumerations as strings rather than integers
+    // since it's more robust against changes in enumeration order and is more
+    // human-readable.  This is a helper for gconf_string_to_enum and vice versa
+    static GConfEnumStringPair schemetype_lookup_table[] = 
+    {
+        { SCHEME_COMPLEMENTS, "COMPLEMENTS" },
+        { SCHEME_SPLIT_COMPLEMENTS, "SPLIT-COMPLEMENTS" },
+        { SCHEME_TRIADS, "TRIADS" },
+        { SCHEME_TETRADS, "TETRADS" },
+        { SCHEME_ANALOGOUS, "ANALOGOUS" },
+        { SCHEME_MONOCHROMATIC, "MONOCHROMATIC" },
+        { 0, NULL }
+    };
+
+    // convenience class to convert between gconf Value classes and basic types
+    // more automatically
+    class Value : public Gnome::Conf::Value
+    {
+        public:
+            Value(const Gnome::Conf::Value& other) : Gnome::Conf::Value(other) {}
+            operator bool() { return get_bool(); }
+            operator int() { return get_int(); }
+            operator Glib::ustring() { return get_string(); }
+            operator double() { return get_float(); }
+            operator float() { return get_float(); }
+    };
+
     const Glib::ustring Conf::APP_DIR = "/apps/agave";
+
     const Glib::ustring Conf::KEY_LAST_SAVE_DIR = Conf::APP_DIR + "/last_save_directory";
     const Glib::ustring Conf::KEY_LAST_COLOR = Conf::APP_DIR + "/last_color";
     const Glib::ustring Conf::KEY_LAST_SCHEME_TYPE = Conf::APP_DIR + "/last_scheme_type";
-
     const Glib::ustring Conf::KEY_WINDOW_WIDTH = Conf::APP_DIR + "/window_width";
     const Glib::ustring Conf::KEY_WINDOW_HEIGHT = Conf::APP_DIR + "/window_height";
+    const Glib::ustring Conf::KEY_FAVORITES_WIDTH = Conf::APP_DIR + "/favorites_width";
+    const Glib::ustring Conf::KEY_PALETTE_EXPANDED = Conf::APP_DIR + "/palette_expanded";
 
     const Glib::ustring Conf::OLD_GCS_CONF_DIR = Glib::build_filename(Glib::get_home_dir(), ".agave");
-    // FIXME: look into getting g_get_user_data_dir() wrapped into glibmm
     const Glib::ustring Conf::GCS_CONF_DIR = Glib::build_filename(g_get_user_data_dir(), "agave");
     const Glib::ustring Conf::FAVORITES_FILE = Conf::GCS_CONF_DIR + "/bookmarks.gpl";
     const Glib::ustring Conf::USER_PALETTE_DIR = Conf::GCS_CONF_DIR + "/palettes";
@@ -65,53 +94,25 @@ namespace gcs
 
     Glib::ustring Conf::get_last_save_dir(void)
     {
-        std::string dir;
-#ifdef HAVE_GCONFMM
-        Glib::RefPtr<Gnome::Conf::Client> ref_client =
-            Gnome::Conf::Client::get_default_client();
-
-        Gnome::Conf::Value val = ref_client->get(KEY_LAST_SAVE_DIR);
-        if (val.get_type() == Gnome::Conf::VALUE_INVALID)
+        Glib::ustring dir;
+        if (!get_value(KEY_LAST_SAVE_DIR, dir) || dir == "")
         {
             dir = Glib::filename_to_uri(Glib::get_current_dir());
         }
-        else
-        {
-            dir = val.get_string();
-        }
-#endif // HAVE_GCONFMM
-
         return dir;
     }
 
 
     void Conf::set_last_save_dir(Glib::ustring uri)
     {
-#ifdef HAVE_GCONFMM
-        Glib::RefPtr<Gnome::Conf::Client> ref_client =
-            Gnome::Conf::Client::get_default_client();
-        if (!uri.empty())
-        {
-            ref_client->set(Conf::KEY_LAST_SAVE_DIR, uri);
-        }
-#endif // HAVE_GCONFMM
+        set_value(KEY_LAST_SAVE_DIR, uri);
     }
 
 
     ColorPtr Conf::get_last_color(void)
     {
-        std::string clr;
-#ifdef HAVE_GCONFMM
-        Glib::RefPtr<Gnome::Conf::Client> ref_client =
-            Gnome::Conf::Client::get_default_client();
-
-        Gnome::Conf::Value val = ref_client->get(KEY_LAST_COLOR);
-        if (val.get_type() != Gnome::Conf::VALUE_INVALID)
-        {
-            clr = val.get_string();
-        }
-        else
-#endif // HAVE_GCONFMM
+        Glib::ustring clr;
+        if (!get_value(KEY_LAST_COLOR, clr))
         {
             clr = "#9DFF00";
         }
@@ -121,38 +122,24 @@ namespace gcs
 
     void Conf::set_last_color(ColorPtr clr)
     {
-#ifdef HAVE_GCONFMM
-        Glib::RefPtr<Gnome::Conf::Client> ref_client =
-            Gnome::Conf::Client::get_default_client();
-        ref_client->set(Conf::KEY_LAST_COLOR, clr->get_hexstring());
-#endif // HAVE_GCONFMM
+        set_value(KEY_LAST_COLOR, clr->get_hexstring());
     }
 
 
     void Conf::set_last_scheme_type(tSchemeType t)
     {
-#ifdef HAVE_GCONFMM
-        Glib::RefPtr<Gnome::Conf::Client> ref_client =
-            Gnome::Conf::Client::get_default_client();
-        ref_client->set(Conf::KEY_LAST_SCHEME_TYPE, t);
-#endif // HAVE_GCONFMM
+        Glib::ustring schemetype_string = gconf_enum_to_string(schemetype_lookup_table, t);
+        set_value(KEY_LAST_SCHEME_TYPE, schemetype_string);
     }
 
 
     tSchemeType Conf::get_last_scheme_type(void)
     {
+        Glib::ustring scheme_type;
         tSchemeType t;
-#ifdef HAVE_GCONFMM
-        Glib::RefPtr<Gnome::Conf::Client> ref_client =
-            Gnome::Conf::Client::get_default_client();
-
-        Gnome::Conf::Value val = ref_client->get(KEY_LAST_SCHEME_TYPE);
-        if (val.get_type() != Gnome::Conf::VALUE_INVALID)
-        {
-            t = static_cast<tSchemeType>(val.get_int());
-        }
-        else
-#endif // HAVE_GCONFMM
+        if (!get_value(KEY_LAST_SCHEME_TYPE, scheme_type) ||
+                !gconf_string_to_enum(schemetype_lookup_table,
+                    scheme_type.c_str(), reinterpret_cast<int*>(&t)))
         {
             t = SCHEME_TRIADS;
         }
@@ -168,24 +155,30 @@ namespace gcs
 
     Glib::ustring Conf::get_conf_dir(void)
     {
+        if (!Glib::file_test(GCS_CONF_DIR, Glib::FILE_TEST_EXISTS) ||
+                !Glib::file_test(GCS_CONF_DIR, Glib::FILE_TEST_IS_DIR))
+        {
+            g_mkdir(GCS_CONF_DIR.c_str(), 0755);
+        }
         return GCS_CONF_DIR;
     }
 
 
+    Glib::ustring Conf::get_user_palette_dir(void)
+    {
+        if (!Glib::file_test(USER_PALETTE_DIR, Glib::FILE_TEST_EXISTS) ||
+                !Glib::file_test(USER_PALETTE_DIR, Glib::FILE_TEST_IS_DIR))
+        {
+            g_mkdir(USER_PALETTE_DIR.c_str(), 0755);
+        }
+        return USER_PALETTE_DIR;
+    }
+
+
     int Conf::get_window_width(void)
     {
         int w;
-#ifdef HAVE_GCONFMM
-        Glib::RefPtr<Gnome::Conf::Client> ref_client =
-            Gnome::Conf::Client::get_default_client();
-        Gnome::Conf::Value val = ref_client->get(KEY_WINDOW_WIDTH);
-
-        if (val.get_type() != Gnome::Conf::VALUE_INVALID)
-        {
-            w = val.get_int();
-        }
-        else
-#endif // HAVE_GCONFMM
+        if (!get_value(KEY_WINDOW_WIDTH, w))
         {
             w = 650;
         }
@@ -195,27 +188,14 @@ namespace gcs
 
     void Conf::set_window_width(int w)
     {
-#ifdef HAVE_GCONFMM
-        Glib::RefPtr<Gnome::Conf::Client> ref_client =
-            Gnome::Conf::Client::get_default_client();
-        ref_client->set(KEY_WINDOW_WIDTH, w);
-#endif // HAVE_GCONFMM
+        set_value(KEY_WINDOW_WIDTH, w);
     }
 
 
     int Conf::get_window_height(void)
     {
         int h;
-#ifdef HAVE_GCONFMM
-        Glib::RefPtr<Gnome::Conf::Client> ref_client =
-            Gnome::Conf::Client::get_default_client();
-        Gnome::Conf::Value val = ref_client->get(KEY_WINDOW_HEIGHT);
-        if (val.get_type() != Gnome::Conf::VALUE_INVALID)
-        {
-            h = val.get_int();
-        }
-        else
-#endif // HAVE_GCONFMM
+        if (!get_value(KEY_WINDOW_HEIGHT, h))
         {
             h = 400;
         }
@@ -225,12 +205,64 @@ namespace gcs
 
     void Conf::set_window_height(int h)
     {
-#ifdef HAVE_GCONFMM
-        Glib::RefPtr<Gnome::Conf::Client> ref_client =
-            Gnome::Conf::Client::get_default_client();
-        ref_client->set(KEY_WINDOW_HEIGHT, h);
-#endif // HAVE_GCONFMM
+        set_value(KEY_WINDOW_HEIGHT, h);
     }
 
 
+    int Conf::get_favorites_width(void)
+    {
+        int w;
+        if (!get_value(KEY_FAVORITES_WIDTH, w))
+        {
+            w = 130;    // initial value
+        }
+        return w;
+    }
+
+
+    void Conf::set_favorites_width(int w)
+    {
+        set_value(KEY_FAVORITES_WIDTH, w);
+    }
+
+
+    bool Conf::get_palette_expanded(void)
+    {
+        bool expanded;
+        if (!get_value(KEY_PALETTE_EXPANDED, expanded))
+        {
+            expanded = true;    // initial value
+        }
+        return expanded;
+    }
+
+
+    void Conf::set_palette_expanded(bool expanded)
+    {
+        set_value(KEY_PALETTE_EXPANDED, expanded);
+    }
+
+
+    template <typename T>
+    bool Conf::get_value(Glib::ustring gconf_key, T& result)
+    {
+#ifdef HAVE_GCONFMM
+        Value val = Gnome::Conf::Client::get_default_client()->get(gconf_key);
+        if (val.get_type() != Gnome::Conf::VALUE_INVALID)
+        {
+            result = static_cast<T>(val);
+            return true;
+        }
+#endif // HAVE_GCONFMM
+        return false;
+    }
+
+
+    template <typename T>
+    void Conf::set_value(Glib::ustring gconf_key, const T& val)
+    {
+#ifdef HAVE_GCONFMM
+        Gnome::Conf::Client::get_default_client()->set(gconf_key, val);
+#endif // HAVE_GCONFMM
+    }
 } // namespace gcs
