@@ -55,7 +55,6 @@ namespace gcs
             m_colormap = get_default_colormap();
             m_colormap->alloc_color(m_black->gdk());
             m_colormap->alloc_color(m_white->gdk());
-            alloc_bg();
             LOG("TextSwatch Created!");
             Glib::RefPtr<Gtk::ActionGroup> refActions = Gtk::ActionGroup::create();
             refActions->add(Gtk::Action::create("CopyColor", Gtk::Stock::COPY),
@@ -80,6 +79,9 @@ namespace gcs
             }
             m_pContextMenu =
                 dynamic_cast<Gtk::Menu*>(m_refUIManager->get_widget("/ColorSwatchPopup"));
+
+            // give the text swatches a bit of a rounded corner look
+            m_cornerRadius = 10;
         }
 
 
@@ -110,10 +112,6 @@ namespace gcs
             // set the size of the swatch.
             set_size_request(m_minSize, m_minSize);
             m_textGC = Gdk::GC::create(window);
-            m_borderGC = Gdk::GC::create(window);
-            m_borderGC->set_line_attributes(m_borderWidth, Gdk::LINE_SOLID,
-                    Gdk::CAP_BUTT, Gdk::JOIN_MITER);
-            m_borderGC->set_foreground(m_black->gdk());
             add_events(Gdk::BUTTON_PRESS_MASK | Gdk::ENTER_NOTIFY_MASK |
                     Gdk::LEAVE_NOTIFY_MASK);
             Glib::RefPtr<Pango::Context> m_refContext = create_pango_context();
@@ -127,6 +125,7 @@ namespace gcs
 
         bool TextSwatch::on_expose_event(GdkEventExpose *e)
         {
+            ColorSwatch::on_expose_event(e);
             if (m_background->get_value() > 65)
             {
                 m_foreground = m_black;
@@ -136,14 +135,6 @@ namespace gcs
                 m_foreground = m_white;
             }
             m_textGC->set_foreground(m_foreground->gdk());
-            redraw();
-            return true;
-        }
-
-
-        void TextSwatch::redraw(void)
-        {
-            ColorSwatch::redraw();
             Glib::RefPtr<Gdk::Window> win = get_window();
             int winWidth, winHeight;
             int textX, textY;
@@ -172,6 +163,7 @@ namespace gcs
 
                 win->draw_layout(m_textGC, textX, textY, pText->get_layout());
             }
+            return true;
         }
 
 
